
#![feature(proc_macro_hygiene, decl_macro)]

#[macro_use]
extern crate rocket;

#[macro_use]
extern crate lazy_static;

use std::path::{PathBuf, Path};
use std::env;
use std::fs;
use std::ffi::OsStr;
use std::collections::HashMap;
use rocket::response::NamedFile;
use rocket::http::ContentType;
use rocket_contrib::templates::Template;
use rocket_upload::MultipartDatas;
use serde::Serialize;

#[derive(Serialize)]
struct DirPath {
    is_file: bool,
    path: String,
    name: String,
}

lazy_static! {
    static ref DIR: PathBuf = Path::new(env::var("FASTCLOUD_DIR").unwrap_or_else(|_| String::from("")).as_str()).to_path_buf();
}

#[get("/serve")]
fn get_root() -> Template {
    let path = DIR.as_path().to_str().unwrap_or("");
    Template::render("dir", directory_structure(path.to_string()))
}

#[get("/serve/<path..>")]
fn get_dir_entry(path: PathBuf) -> Template {
    let dir = DIR.as_path().join(path);
    let path = dir.to_str().unwrap_or("");
    Template::render("dir", directory_structure(path.to_string()))
}

#[post("/dir/<path..>")]
fn create_dir(path: PathBuf) {
    let dir = DIR.as_path().join(path);
    fs::create_dir_all(dir).unwrap();
}

#[post("/del/<path..>")]
fn delete(path: PathBuf) {
    let dir = DIR.as_path().join(path);
    if dir.is_file() {
        fs::remove_dir_all(dir).unwrap();
    } else {
        fs::remove_file(dir).unwrap();
    }
}

fn directory_structure(path: String) -> HashMap<&'static str, HashMap<i32, DirPath>> {
    let dir = Path::new(path.as_str());
    let paths = fs::read_dir(dir).unwrap()
        .map(|x| x.unwrap().path().into_os_string().into_string().unwrap())
        .enumerate()
        .map(|x| (x.0 as i32, DirPath { 
            is_file: Path::new(&x.1).is_file(), 
            path: Path::new(&x.1).strip_prefix(DIR.clone().into_os_string().into_string().unwrap()).unwrap().to_str().unwrap().to_string(),
            name: Path::new(&x.1).file_name().unwrap().to_str().unwrap().to_string(),
        }))
        .collect::<HashMap<i32, DirPath>>();
    let mut context: HashMap<&str, HashMap<i32, DirPath>> = HashMap::new();
    let parent = DirPath {
        is_file: false,
        path: dir.strip_prefix(DIR.clone().into_os_string().into_string().unwrap()).unwrap().parent().unwrap_or_else(|| Path::new("")).to_str().unwrap().to_string(),
        name: dir.parent().unwrap().file_name().unwrap_or_else(|| OsStr::new("")).to_str().unwrap().to_string(),
    };
    let mut pt = HashMap::new();
    pt.insert(0, parent);
    context.insert("ctx", paths);
    context.insert("parent", pt);
    context
}


#[get("/file/<path..>")]
fn file_get(path: PathBuf) -> Option<NamedFile> {
    let dir = DIR.as_path().join(path);
    rocket::response::NamedFile::open(dir.as_path()).ok()
}

#[post("/upload/<path..>", data = "<data>")]
fn upload_file(path: PathBuf, _content_type: &ContentType, data:MultipartDatas) {
    let dir = DIR.as_path().join(path);
    for f in data.files {
        if !Path::new(&format!("{}/{}", dir.to_str().unwrap(), f.filename)).exists() {
            f.persist(&dir);
        }
        for i in 0.. {
            if !Path::new(&format!("{}/{}.{}", dir.to_str().unwrap(), f.filename, i)).exists() {
                f.persist(&dir);
                break
            }
        }
    }
}

fn main() {
    rocket::ignite()
        .attach(Template::fairing())
        .mount("/", routes![get_root, get_dir_entry, create_dir, file_get, upload_file, delete]).launch();
}
